#!/usr/bin/perl
#
# Copyright (C) 2007 Peteris Krumins (peter@catonmat.net)
# http://www.catonmat.net  -  good coders code, great reuse
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

package sites::reddit;

use warnings;
use strict;

use HTML::TreeBuilder;
use HTML::Entities;
use Time::ParseDate;
use POSIX;

use base 'sites::scraper';

#
# scraper plugin for reddit.com
#
# This plugin is a part of picurls.com website data miner.
# More about it at:
# http://www.catonmat.net/blog/making-of-picurls-popurls-for-pictures-part-one
#

use constant POSTS_PER_PAGE => 25;

#
# site_name
#
# returns short name for the site
#
sub site_name { 'reddit' }

#
# get_page_url
#
# given a page number, returns url of that page
#
sub get_page_url {
    my ($self, $page) = @_;

    my $offset = POSTS_PER_PAGE * ($page - 1);

    my $url;
    if (exists $self->{vars}->{subreddit}) {
        $url = "http://$self->{vars}->{subreddit}.reddit.com/"
    }
    else {
        $url = "http://reddit.com/";
    }
    $url .= "?offset=$offset";

    return $url;
}

#
# get_posts
#
# taken from redditmedia.com website generator:
# http://www.catonmat.net/blog/designing-reddit-media-website/
#
#
# subroutine takes HTML content of reddit's page and returns an array
# of hashes containing information about each post on the page.
#
sub get_posts {
    my ($self, $content) = @_;

    my $tree = HTML::TreeBuilder->new;
    $tree->parse($content);
    $tree->eof;

    # if we look how the reddit is made in FireBug, we see that each link
    # is a row of a big HTML table.
    # each row has 'class' attribute named 'evenRow' or 'oddRow'
    #
    my (@posts, %post_entry);
    my @trs = $tree->look_down("_tag" => "tr", "class" => qr{evenRow|oddRow});
    foreach my $tr (@trs) {
        my $link = $tr->look_down("_tag" => "a", "id" => qr{title\d+});
        unless (defined $link) {
            # if link is not defined, it means that we have the row containing user info
            # and id of reddit url for comments, and score
            my ($ulink, $clink) = $tr->look_down("_tag" => "a");
            next unless (defined $ulink and defined $clink);

            my $user = $ulink->as_text;
            $user =~ s/\s+$//g;
            $post_entry{user} = $user;

            my $chref = $clink->attr('href');
            next unless $chref;
            if ($chref =~ m{/info/([^/]+)/comments}) {
                $post_entry{id} = $1;
            }

            my $score_span = $tr->look_down("_tag" => "span", id => qr{score\d+});
            if ($score_span) {
                my $score_text = $score_span->as_text;
                if ($score_text =~ /(\d+) point/) {
                    $post_entry{score} = $1;

                    my $post_time_info = $score_span->right;
                    if ($post_time_info =~ /posted (.+) by/) {
                        my $unix_time  = parsedate($1);
                        my $human_time = strftime("%Y-%m-%d %H:%M:%S", localtime $unix_time);
                        $post_entry{unix_time}  = $unix_time;
                        $post_entry{human_time} = $human_time;
                    }
                }

                
            }

            unless (exists $post_entry{score}) {
                # could be that the entry was posted less than hour ago,
                # then it has no score visible
                $post_entry{score} = 0;
            }
  
            push @posts,  { %post_entry };
            next;
        }

        # get the title, strip leading spaces
        my $title = decode_entities($link->as_text);
        $title =~ s/^\s+//g;

        my $url = $link->attr('href');

        $post_entry{title} = $title;
        $post_entry{url}   = $url;
    }

    $tree->delete;
    return @posts;    
}


1;
